//////////////////////// STATE CLASS /////////////////////

/*
This class records what content has been presented and what is currently
onscreen. It also records the state of any variables that are essential
to the reading. If all the data in the State class is saved to the
Flash Shared Object or to a file, an interrupted reading can be resumed
exactly where it was left off by restoring the saved State data.

Structure of class:

// Currently displayed group and unit
State.current.contents[0...].group = groupName
                            .units[0...] = unitName

// Groups and units within groups presented
State.presented.groups.groupName.allUnits = [array of unitNames]
State.presented.groups.groupName.lastUnit = unitName
// History of all units displayed; duplicate entries included
// (as a result of backtracking)
State.presented.units = [array of unitNames]

*/

package xal {

//import flash.xml.XMLDocument;
import xal.Utilities;


///////////////////// CLASS DEFINITION //////////////////

public class State {


// VARIABLES

// Utilities class
private var utilities:Utilities;
private var _current:Object;
private var _presented:Object;
private var _variables = new Object();


// GETTERS

public function get current():Object {
	return _current;
}

public function get presented():Object {
	return _presented;
}

public function get variables():Object {
	return _variables;
}


// CONSTRUCTOR

public function State(groupNames:Array):void {
	
	utilities = new Utilities();
	
	// current.contents is an array in case in the future there is the need to
	// allow more than one group or unit to be current
	// at the same time
	_current = {contents : new Array()};
	_current.contents[0] = {
							group : "",
							units : new Array()
							};
	_presented = {groups : new Object(),
				  allGroups : new Array(),
				  units : new Array()
				  };
	initGroups(groupNames);
//	// Add each group name as a property of "groups" and create empty properties
//	// to record units presented for each group
//	for (var i:int = 0; i < groupNames.length; i ++) {
//		_presented.groups[groupNames[i]] = {allUnits : new Array(),
//										 lastUnit : ""
//										 };
//	}
	
}

// Add each group name as a property of "groups" and create empty properties
// to record units presented for each group
public function initGroups(groupNames:Array) {
	for (var i:int = 0; i < groupNames.length; i ++) {
		_presented.groups[groupNames[i]] = {allUnits : new Array(),
										 lastUnit : ""
										 };
	}
}

// Set group and unit names as current elements and add them to
// the array of presented elements
public function addContent(groupName:String, unitName:String):void {
	// Add names to current
	_current.contents[0].group = groupName;
	_current.contents[0].units[0] = unitName;
	// Add group name to array, if it isn't already present
	if (!utilities.isInArray(_presented.allGroups, groupName)) {
		_presented.allGroups.push(groupName);
	}
	// Add unit to array for group, if it isn't already present
	if (!utilities.isInArray(_presented.groups[groupName].allUnits, unitName)) {
		_presented.groups[groupName].allUnits.push(unitName);
	}
	// Record last unit presented in group
	_presented.groups[groupName].lastUnit = unitName;
	// Record history of all units presented
	_presented.units.push(unitName);
}

// Determine whether any units in the specified groups have been presented;
// accepts an array of group names as second parameter;
// if quantity = "some" returns true if some of the groups have been presented;
// if quantity = "all" returns true if all of the groups have been presented.
public function wasPresented(quantity:String, groupNamesArray:Array):Boolean {
	var presented:Boolean = false;
	// Check each passed groupName against property names of "_presented"
	// Each passed groupName
	for (var i:int = 0; i < groupNamesArray.length; i ++) {
		presented = false;
		// Each group that has been presented
		for (var j:int = 0; j < _presented.allGroups.length; j ++) {
			if (_presented.allGroups[j] == groupNamesArray[i]) {
				presented = true;
				// If only one match must be found, abort when it's found
				if (quantity == "some") {
					if (presented === true) {
						return true;
					}
				}
			}
		}
		// If all groupNames must match, abort when first name
		// is found that doesn't match
		if (quantity == "all") {
			if (presented === false) {
				return false;			
			}
		}
	}
	return presented;
}

//// Returns an array of the names
//// of all groups that have been presented
//public function groupsPresented():Array {
//	var groupsArray = new Array();
//	// Get each groupName
//	for (var groupName:String in _presented.groups) {
//		groupsArray.push(groupName);
//	}
//	return groupsArray;
//}

// Set a variable that affects how content is accessed and displayed;
// this var becomes part of the state;
// values are restricted to strings so they can be easily
// serialized or saved to a file;
// These variables can be read at runtime by the functions defined
// in the XML file
public function setVar(name:String, value:String):void {
	_variables[name] = value;
}


}

}