﻿///////////// XLIT CLASS ////////////////////

// Version: .1.1
// Date: 5/11/2008
// Language: ActionScript 3.0
// Author: Robert Kendall
// Requires: State.as, Utilities.as

// Class for reading content of XML file into structured data objects.
// Documentation for this library available at http://www.wordcircuits.com/xlit


package xal {

// These functions let you change depth of clips
import flash.xml.*;
import flash.display.*;
import flash.net.*;
import flash.events.*;
import flash.errors.*;

import xal.State;
import xal.Utilities;

/////////////////////////////// CLASS DEFINITION //////////////////////////////////

public class Xlit extends Sprite {


// VARIABLES

// State Class
// Stores state data about what was/is presented
private var _currentState:State;

private var utilities:Utilities;

// Contents of XML file
private var _rawXML:XML;

private var _xmlLoader:URLLoader;

// Stores all content objects
private var _contents:Object = new Object();
//private var _state:Object = new Object();

// Pages parsed from XML file
private var _pageTexts:Array = new Array();

// Index no. of currently displayed page
private var _currentPageInd:Number = 0;

private var _contentsTag:String = "contents";

private var _isXmlLoaded:Boolean = false;

// GETTERS

public function get currentState():State {
	return _currentState;
}

public function get pageTexts():Array {
	return _pageTexts;
}

public function get contents():Object {
	return _contents;
}

public function get isXmlLoaded():Boolean {
	return _isXmlLoaded;
}


// SETTERS

public function set contents(contents:Object):void {
	_contents = contents;
}


// CONSTRUCTOR

public function Xlit(filename:String):void {
	utilities = new Utilities();
	// List of all groups
	_contents["_groups"] = new Array();
	loadXML(filename);
	//initState();
}


//////////////////////////////////// FUNCTIONS ///////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////// LOAD AND PARSE XML DATA //////////////////////////////////////////


// Loads all content units from XML file
public function loadXML(fileName:String):void {
	_xmlLoader = new URLLoader();
	_xmlLoader.addEventListener(Event.COMPLETE, xmlLoaded);
	_xmlLoader.addEventListener(IOErrorEvent.IO_ERROR, errorLoadingXML);
	var request:URLRequest = new URLRequest(fileName);
	_xmlLoader.load(request);
//	return _contents;
}

// Executed when XML file has finished loading
private function xmlLoaded(event:Event):void {
	try {
		_rawXML = new XML(_xmlLoader.data);
		_rawXML.ignoreWhite = true;
	} catch(error:Error) {
		trace("Error: " + error.message);
		return;
    }
	parseXMLContents();
}

private function errorLoadingXML(event:Event) {
	trace("Couldn't load XML document");
}

// Parse XML and add contents to contents array
// Called by loadXML
//
// Structure of contents:
// contents[groupName][unitName].content = unitConent
// contents[groupName]._units[0] = unitName
// The _units array allows units to be accesed by index number
// instead of name
private function parseXMLContents():void {
	
	// Each group
	for each (var group:XML in _rawXML..group) {
		
		var unitInd:int = 0;
		var groupName:String  = group.attribute("name");	
		
		// To create unit object
		function Unit(unitContent:Array) {
			this.content = unitContent;
		}
		// To attach getContent() to each unit object
		Unit.prototype.getContent = function():String {
			var unitText = getUnitText(this.content);
			return unitText;
		}
		
		// To create group object from XML object
		function Group(group:XML) {
			var groupName:String  = group.attribute("name");
			this._name = groupName;
			this._units = new Array();
			// Each unit
			for each (var unit:XML in group..unit) {
				var unitName:String = unit.attribute("name");
				var unitContent:Array = parseUnit(unit);
				// If unit doesn't have a name, assign it a generic one
				if (!unitName) {
					unitName = groupName + "_unit_" + (unitInd + 1).toString();
				}
				// Create unit
				this[unitName] = new Unit(unitContent);
				// Add name to _units array
				this._units.push(unitName);
				unitInd ++;
			}
		}
		// To attach getUnit() to each group
		Group.prototype.getUnit = function(position:String):String {
			var unitStr = getUnitForGroup(this._name, position)
			return unitStr;
		}
		
		// Create group
		contents[groupName] = new Group(group);
		// Add each group name to array
		contents._groups.push(groupName);
		
	}
	
	_currentState = new State(contents._groups);
	// For notifying other classes that XML content is ready
	_isXmlLoaded = true;
	
}
	
// Parse contents of unit node and store in array
// Called by parseXMLContents()
private function parseUnit(unit:XML):Array {
	
	var unitArray:Array = new Array();
	
	for (var unitInd:int = 0; unitInd < unit.children().length(); unitInd ++) {
		
		var nodeName:String = unit.children()[unitInd].name();
		var statementNodes:XMLList;
		unitArray[unitInd] = new Object();
		
		switch (nodeName) {
			// Parse Text elements
			case "text":
				unitArray[unitInd]["type"] = nodeName;
				unitArray[unitInd]["data"] = unit.children()[unitInd].text();
				break;
			// Parse Choose elements, which specify conditions for displaying content
			case "choose":
				statementNodes = unit.children()[unitInd].children();
				unitArray[unitInd]["type"] = nodeName;
				unitArray[unitInd]["statements"] = new Array();
				
				// Parse each statement (when or otherwise) and put it in object with these attributes
				// Label (When or Otherwise)
				// Parameters
				// Content
				for (var statementInd:int = 0; statementInd < statementNodes.length(); statementInd ++) {
					var contentName:String = "";
					var contentValue:String = "";
					var whenAttributes:XMLList;
					var statementName:String = statementNodes[statementInd].name().toString();
					
					unitArray[unitInd].statements[statementInd] = new Object();
					
					// Get attribute values for "when" statement
					if (statementName == "when") {
						// Add Parameters (parsed from element attributes)
						whenAttributes = statementNodes[statementInd].attributes();
						unitArray[unitInd].statements[statementInd]["parameters"] = new Object();
						for (var attribInd:int = 0; attribInd < whenAttributes.length(); attribInd ++) {  
							var attribName:String = whenAttributes[attribInd].name();
							var attribValue:String = whenAttributes[attribInd];   
							unitArray[unitInd].statements[statementInd].parameters[attribName] = attribValue;
						}
					}
					
					// Add Label to array
					unitArray[unitInd].statements[statementInd]["label"] = statementName;
					// Add Content
					contentName = statementNodes[statementInd].children()[0].name();
					contentValue = statementNodes[statementInd].children()[0];
					// In case of empty nodes
					if (contentValue == null) {
						contentValue = "";
					}
					unitArray[unitInd].statements[statementInd].content = {type : contentName, data : contentValue};
				}
				break;
		}
	}
	return unitArray;
}


///////////////////////////////// GET CONTENT FROM DATA STRUCTURES //////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////


// Get the text of the next or previous unit in the group;
// the "position" parameter is either "first," "last," "next," "previous," or a number;
// if it is a positive number, return the unit in that position; if negative, count back
// from the end, with 0 being the last unit;
// if the last unit in the group is the current unit and position is "next", return an empty string;
// Called by getUnit()
private function getUnitForGroup(groupName:String, position:String):String {
	
	var textStr:String = "";
	// Index of last unit in group
	var finalUnitInd:Number = contents[groupName]._units.length - 1;
	var unitInd:Number;
	var newUnit:String;
	var currentUnit:String; 
	var currentUnitInd:Number;
	var positionNum:int;
	
	// If parameter specifies the first unit in the group,
	// or if a groupName is passed that has never been presented,
	// set index to -1 so first unit parsed will be 0
	if (position ==  "first" || !currentState.wasPresented("some", [groupName])) {
		currentUnitInd = -1;
	} else {
		currentUnit = currentState.presented.groups[groupName].lastUnit;
		currentUnitInd = getUnitInd(groupName, currentUnit);
	}
	
	if (position == "first" || position == "next") {
		// Check each unit until one is found that isn't empty;
		// some units may be empty because the text may have been removed conditionally
		for (unitInd = currentUnitInd; unitInd < finalUnitInd; unitInd ++) {
			newUnit = contents[groupName]._units[unitInd + 1];
			textStr = contents[groupName][newUnit].getContent();
			if (textStr != "") {
				break;
			} else {
				continue;
			}
		}
	} else if (position == "previous") {
		// Check each previous unit until one is found that isn't empty;
		for (unitInd = currentUnitInd; unitInd > 0; unitInd --) {
			newUnit = contents[groupName]._units[unitInd - 1];
			textStr = contents[groupName][newUnit].getContent();
			if (textStr != "") {
				break;
			} else {
				continue;
			}
		}
	// Other position parameters
	// Don't check each unit, just go to the one specified
	} else {
		// Get last text unit
		if (position == "last") {
			unitInd = contents[groupName]._units.length - 1;
		} else {
			// Get a text unit specified by a number, if position is a number
			if (!isNaN(parseInt(position, 10))) {
				positionNum = parseInt(position, 10);
				// Positive number
				if (positionNum > 0) {
					unitInd = positionNum - 1;
				// 0 or negative numbers count back from end
				} else {
					unitInd = contents[groupName]._units.length - 1 + positionNum;
				}
			} else {
				trace("Unrecognized parameter passed to getUnitForGroup()");
			}
		}
		// Get text string
		newUnit = contents[groupName]._units[unitInd];
		textStr = contents[groupName][newUnit].getContent();
	}
	// If new unit was found
	if (textStr != "") {
		// Set new unit as current unit and mark it as having been presented
		currentState.addContent(groupName, newUnit);
	}
	return textStr;
}

// Generate unit text from content and behaviors stored in contents
private function getUnitText(unitContent:Array):String {
	var unitStr:String = "";
	var contentType:String = "";
	var currentStatement:Object = new Object();
	var conditionFound:Boolean = false;
	var i;
	var j;
	for (i = 0; i < unitContent.length; i ++) {
		contentType = unitContent[i].type;
		switch (contentType) {
			case "text":
				unitStr += unitContent[i].data;
				break;
			case "choose":
				// Parse and execute statements (when or otherwise) in Choose
				var countStatements:Number = unitContent[i].statements.length;
				for (j = 0; j < countStatements; j ++) {
					currentStatement = unitContent[i].statements[j];
					switch (currentStatement.label) {
						case "when":
							// If there's a var parameter
							if (currentStatement.parameters.variable != undefined) {
								var varName = currentStatement.parameters.variable;
								var varValue = currentStatement.parameters.value;
								// Convert text values to Boolean if necessary
								if (varValue == "true") {
									varValue = true;
								} else if (varValue == "false") {
									varValue = false;
								}
								if (currentState.variables[varName] === varValue) {
									unitStr += currentStatement.content.data;
									conditionFound = true;
								}
							// If there's a condition parameter
							} else if (currentStatement.parameters.condition != undefined) {
								// If the specified group has been presented, condition is true
								if (currentState.wasPresented("all", [currentStatement.parameters.group])) {
									unitStr += currentStatement.content.data;
									conditionFound = true;
								}
							}
							break;
						case "otherwise":
							unitStr += currentStatement.content.data;
							break;
					}
					// If condition is met, skip succeeding statements
					if (conditionFound === true) {
						break;
					}
				}
				break;
		}
	}
	return unitStr;
}


/////////////////////////////// INITIALIZATION ////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////


// Initialize the state object that trackes which content is current (ie, currently displayed)
public function initState():void {
	_currentState = new State(_contents._groups);
}


/////////////////// UTILITY FUNCTIONS ////////////////////////////////
//////////////////////////////////////////////////////////////////////


// Get index number of unit from unit name; returns -1 if name not found
private function getUnitInd(groupName:String, unitName:String):Number {
	var unitsCount = _contents[groupName]._units.length;
	for (var i:int = 0; i < unitsCount; i ++) {
		if (_contents[groupName]._units[i] == unitName) {
			return i;
		}
	}
	// If not found
	return -1;
}


}

}